@@ -52,8 +52,9 @@ class Geocoder(object):
     """
 
     GEOCODE_QUERY_URL = 'https://maps.google.com/maps/api/geocode/json?'
+    USER_AGENT = 'pygeocoder/' + VERSION + ' (Python)'
 
-    def __init__(self, api_key = None, client_id=None, private_key=None):
+    def __init__(self, api_key=None, client_id=None, private_key=None):
         """
         Create a new :class:`Geocoder` object using the given `client_id` and
         `private_key`.
@@ -67,8 +68,8 @@ class Geocoder(object):
         :param private_key: Google Maps Premier API key
         :type client_id: string
 
-        Google Maps API Premier users can provide his key to make 100,000 requests
-        a day vs the standard 2,500 requests a day without a key
+        Google Maps API Premier users can provide his key to make 100,000
+        requests a day vs the standard 2,500 requests a day without a key
 
         """
         self.api_key = api_key
@@ -175,12 +176,13 @@ class Geocoder(object):
         :rtype: (dict or array)
 
         """
-        request = requests.Request('GET',
-                url = Geocoder.GEOCODE_QUERY_URL,
-                params = params,
-                headers = {
-                    'User-Agent': 'pygeocoder/' + VERSION + ' (Python)'
-                })
+        request = requests.Request(
+            'GET',
+            url=Geocoder.GEOCODE_QUERY_URL,
+            params=params,
+            headers={
+                'User-Agent': Geocoder.USER_AGENT
+            })
 
         if self and self.client_id and self.private_key:
             self.add_signature(request)
@@ -204,13 +206,26 @@ class Geocoder(object):
         return response_json['results']
 
     def add_signature(self, request):
-        request.params['client'] = str(self.client_id)
-        decoded_key = base64.urlsafe_b64decode(self.private_key)
-        url = urlparse(request.url)
-        url_to_sign = url.path + "?" + url.query
-        signature = hmac.new(decoded_key, url_to_sign, hashlib.sha1)
-        encoded_signature = base64.urlsafe_b64encode(signature.digest())
-        request.params['signature'] = encoded_signature
+        """
+        Add the client_id and signature parameters to the URL
+        Based on http://gmaps-samples.googlecode.com/svn/trunk/urlsigning/urlsigner.py
+        See https://developers.google.com/maps/documentation/business/webservices/auth#signature_examples
+        :return: requests.Request object of type 'GET'
+        """
+        inputStr = request.prepare().url + '&client=' + self.client_id
+        url = urlparse(inputStr)
+        urlToSign = url.path + "?" + url.query
+        decodedKey = base64.urlsafe_b64decode(self.private_key)
+        signature = hmac.new(decodedKey, urlToSign, hashlib.sha1)
+        encodedSignature = base64.urlsafe_b64encode(signature.digest())
+        urlSigned = inputStr + '&signature=' + encodedSignature
+        request = requests.Request(
+            'GET',
+            url=urlSigned,
+            headers={
+                'User-Agent': Geocoder.USER_AGENT
+            })
+
 
 if __name__ == "__main__":
     import sys
